/* Testing a few points about parallel arrays with dynamic shapes */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define TWOPI      6.2831853f

int Q = 4, N = 64;	/* Shape parameters */

shape [][][]pair;	/* Will be [Q][N][N]	*/

/* For future global parallel variables.  In effect, we will have	*/
/* float:pair x0[3], w							*/

float:pair *x0, *w;

/* Allocate global parallel variables, pair must have been allocated	*/

void create_globals()
 { int fsz, vsz;
   fsz = boolsizeof(float:pair);  vsz = 3*fsz;
   x0 = palloc(pair, vsz);
   w  = palloc(pair, fsz);
 }

/* Try a few things */

void main()
 { int i,k;  float a, b, c, d, e, f;
   a = TWOPI;  b = a/2.0f;  c = sqrt(a);
   allocate_shape(&pair, 3, Q, N, N);
   create_globals();
   i = 5;  k = 7;
   /* This is scalar access, so we don't need   with (pair) ... */
   printf("Storing individual values to w.\n");
   [0][k][0]*w = a;  [0][k][k]*w = b;
   printf("Reading individual values from w.\n");
   d = [0][k][0]*w;  e = [0][k][k]*w;
   printf("2PI = %9.5f,  PI = %9.5f\n", d, e);

   printf("Storing individual values to [0][k][0]x0.\n");
   [0][k][0]x0[0] = a;  [0][k][0]x0[1] = b;  [0][k][0]x0[2] = c;
   printf("Reading individual values from [0][k][0]x0.\n");
   d = [0][k][0]x0[0];  e = [0][k][0]x0[1];  f = [0][k][0]x0[2];
   printf("2PI = %9.5f,  PI = %9.5f,  sqrt(2PI) = %9.5f\n", d, e, f);

   printf("Storing individual values to [0][k][i]x0.\n");
   [0][k][i]x0[0] = a;  [0][k][i]x0[1] = b;  [0][k][i]x0[2] = c;
   printf("Reading individual values from [0][k][0]x0.\n");
   d = [0][k][i]x0[0];  e = [0][k][i]x0[1];  f = [0][k][i]x0[2];
   printf("2PI = %9.5f,  PI = %9.5f,  sqrt(2PI) = %9.5f\n",
	[0][k][i]x0[0], [0][k][i]x0[1],    [0][k][i]x0[2]);
 }
